# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["SessionRetrieveResponse"]


class SessionRetrieveResponse(BaseModel):
    configuration: Optional[object] = None
    """The configuration settings for the session."""

    created_at: Optional[datetime] = None
    """The timestamp when the session was created."""

    credits_used: Optional[float] = None
    """The number of credits consumed by the session."""

    duration: Optional[int] = None
    """The duration of the session in seconds."""

    playground: Optional[bool] = None
    """Whether this is a playground session."""

    proxy_bytes: Optional[int] = None
    """The number of bytes transferred through the proxy."""

    session_id: Optional[str] = None
    """The unique identifier of the session."""

    status: Optional[str] = None
    """The current status of the session."""

    steps: Optional[List[object]] = None
    """Array of steps executed in the session."""

    tags: Optional[object] = None
    """Tags associated with the session."""

    team_id: Optional[str] = None
    """The team ID associated with the session."""

    tokens: Optional[object] = None
    """Token usage information."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, Iterable, cast
from typing_extensions import Literal

import httpx

from .all import (
    AllResource,
    AsyncAllResource,
    AllResourceWithRawResponse,
    AsyncAllResourceWithRawResponse,
    AllResourceWithStreamingResponse,
    AsyncAllResourceWithStreamingResponse,
)
from .mouse import (
    MouseResource,
    AsyncMouseResource,
    MouseResourceWithRawResponse,
    AsyncMouseResourceWithRawResponse,
    MouseResourceWithStreamingResponse,
    AsyncMouseResourceWithStreamingResponse,
)
from ...types import (
    session_goto_params,
    session_paste_params,
    session_create_params,
    session_scroll_params,
    session_upload_file_params,
    session_drag_and_drop_params,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ..._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .keyboard import (
    KeyboardResource,
    AsyncKeyboardResource,
    KeyboardResourceWithRawResponse,
    AsyncKeyboardResourceWithRawResponse,
    KeyboardResourceWithStreamingResponse,
    AsyncKeyboardResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .clipboard import (
    ClipboardResource,
    AsyncClipboardResource,
    ClipboardResourceWithRawResponse,
    AsyncClipboardResourceWithRawResponse,
    ClipboardResourceWithStreamingResponse,
    AsyncClipboardResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from .agent.agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .recordings.recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)
from ...types.session_copy_response import SessionCopyResponse
from ...types.session_goto_response import SessionGotoResponse
from ...types.session_paste_response import SessionPasteResponse
from ...types.session_create_response import SessionCreateResponse
from ...types.session_scroll_response import SessionScrollResponse
from ...types.shared.success_response import SuccessResponse
from ...types.session_retrieve_response import SessionRetrieveResponse
from ...types.session_list_pages_response import SessionListPagesResponse
from ...types.session_upload_file_response import SessionUploadFileResponse
from ...types.session_drag_and_drop_response import SessionDragAndDropResponse
from ...types.session_retrieve_downloads_response import SessionRetrieveDownloadsResponse

__all__ = ["SessionsResource", "AsyncSessionsResource"]


class SessionsResource(SyncAPIResource):
    @cached_property
    def all(self) -> AllResource:
        return AllResource(self._client)

    @cached_property
    def recordings(self) -> RecordingsResource:
        return RecordingsResource(self._client)

    @cached_property
    def mouse(self) -> MouseResource:
        return MouseResource(self._client)

    @cached_property
    def keyboard(self) -> KeyboardResource:
        return KeyboardResource(self._client)

    @cached_property
    def clipboard(self) -> ClipboardResource:
        return ClipboardResource(self._client)
    @cached_property
    def agent(self) -> AgentResource:
        return AgentResource(self._client)

    @cached_property
    def with_raw_response(self) -> SessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return SessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return SessionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        browser: session_create_params.Browser | Omit = omit,
        integrations: Iterable[session_create_params.Integration] | Omit = omit,
        session: session_create_params.Session | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionCreateResponse:
        """
        Allocates a new browser session for the user, with optional configurations for
        ad-blocking, captcha solving, proxy usage, and idle timeout.

        Args:
          browser: Browser-specific configurations.

          integrations: Array of integrations to load in the browser session. Integrations must be
              previously created using the Integrations API.

          session: Session-related configurations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/sessions",
            body=maybe_transform(
                {
                    "browser": browser,
                    "integrations": integrations,
                    "session": session,
                },
                session_create_params.SessionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionCreateResponse,
        )

    def retrieve(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionRetrieveResponse:
        """
        Retrieves detailed information about a specific browser session.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v1/sessions/{session_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionRetrieveResponse,
        )

    def delete(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuccessResponse:
        """
        Deletes the browser session associated with the provided browser session ID.
        Requires a valid API key for authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._delete(
            f"/v1/sessions/{session_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponse,
        )

    def copy(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionCopyResponse:
        """
        Copies the currently selected text to the clipboard

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/copy",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionCopyResponse,
        )

    def drag_and_drop(
        self,
        session_id: str,
        *,
        end_x: int,
        end_y: int,
        start_x: int,
        start_y: int,
        button: Literal["left", "middle", "right"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionDragAndDropResponse:
        """
        Performs a drag and drop operation from start coordinates to end coordinates

        Args:
          end_x: Ending X coordinate

          end_y: Ending Y coordinate

          start_x: Starting X coordinate

          start_y: Starting Y coordinate

          button: Mouse button to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/drag-and-drop",
            body=maybe_transform(
                {
                    "end_x": end_x,
                    "end_y": end_y,
                    "start_x": start_x,
                    "start_y": start_y,
                    "button": button,
                },
                session_drag_and_drop_params.SessionDragAndDropParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionDragAndDropResponse,
        )

    def goto(
        self,
        session_id: str,
        *,
        url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionGotoResponse:
        """
        Navigates the browser session to the specified URL

        Args:
          url: The URL to navigate to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/goto",
            body=maybe_transform({"url": url}, session_goto_params.SessionGotoParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionGotoResponse,
        )

    def list_pages(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionListPagesResponse:
        """
        Retrieves a list of pages associated with a specific browser session.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v1/sessions/{session_id}/pages",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionListPagesResponse,
        )

    def paste(
        self,
        session_id: str,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionPasteResponse:
        """
        Pastes text at the current cursor position

        Args:
          text: Text to paste

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/paste",
            body=maybe_transform({"text": text}, session_paste_params.SessionPasteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionPasteResponse,
        )

    def retrieve_downloads(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionRetrieveDownloadsResponse:
        """Retrieves metadata of files downloaded during a browser session.

        Requires a
        valid API key for authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v1/sessions/{session_id}/downloads",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionRetrieveDownloadsResponse,
        )

    def retrieve_screenshot(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Takes a screenshot of the current browser session and returns it as an image.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        extra_headers = {"Accept": "image/png", **(extra_headers or {})}
        return self._get(
            f"/v1/sessions/{session_id}/screenshot",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def scroll(
        self,
        session_id: str,
        *,
        delta_y: int,
        x: int,
        y: int,
        delta_x: int | Omit = omit,
        steps: int | Omit = omit,
        use_os: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionScrollResponse:
        """
        Performs a scroll action at the specified coordinates

        Args:
          delta_y: Vertical scroll amount (positive is down, negative is up)

          x: X coordinate

          y: Y coordinate

          delta_x: Horizontal scroll amount (positive is right, negative is left)

          steps: Number of steps to break the scroll into for smoother scrolling

          use_os: Whether to use the OS scroll or the Playwright scroll

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v1/sessions/{session_id}/scroll",
            body=maybe_transform(
                {
                    "delta_y": delta_y,
                    "x": x,
                    "y": y,
                    "delta_x": delta_x,
                    "steps": steps,
                    "use_os": use_os,
                },
                session_scroll_params.SessionScrollParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionScrollResponse,
        )

    def upload_file(
        self,
        session_id: str,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionUploadFileResponse:
        """
        Upload files directly to a browser session for use with web forms and file
        inputs.

        Files are saved to the session's uploads directory and can be referenced in CDP
        commands.

        Args:
          file: File to upload to the browser session

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/v1/sessions/{session_id}/uploads",
            body=maybe_transform(body, session_upload_file_params.SessionUploadFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionUploadFileResponse,
        )


class AsyncSessionsResource(AsyncAPIResource):
    @cached_property
    def all(self) -> AsyncAllResource:
        return AsyncAllResource(self._client)

    @cached_property
    def recordings(self) -> AsyncRecordingsResource:
        return AsyncRecordingsResource(self._client)

    @cached_property
    def mouse(self) -> AsyncMouseResource:
        return AsyncMouseResource(self._client)

    @cached_property
    def keyboard(self) -> AsyncKeyboardResource:
        return AsyncKeyboardResource(self._client)

    @cached_property
    def clipboard(self) -> AsyncClipboardResource:
        return AsyncClipboardResource(self._client)
    @cached_property
    def agent(self) -> AsyncAgentResource:
        return AsyncAgentResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncSessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncSessionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        browser: session_create_params.Browser | Omit = omit,
        integrations: Iterable[session_create_params.Integration] | Omit = omit,
        session: session_create_params.Session | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionCreateResponse:
        """
        Allocates a new browser session for the user, with optional configurations for
        ad-blocking, captcha solving, proxy usage, and idle timeout.

        Args:
          browser: Browser-specific configurations.

          integrations: Array of integrations to load in the browser session. Integrations must be
              previously created using the Integrations API.

          session: Session-related configurations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/sessions",
            body=await async_maybe_transform(
                {
                    "browser": browser,
                    "integrations": integrations,
                    "session": session,
                },
                session_create_params.SessionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionCreateResponse,
        )

    async def retrieve(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionRetrieveResponse:
        """
        Retrieves detailed information about a specific browser session.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v1/sessions/{session_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionRetrieveResponse,
        )

    async def delete(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SuccessResponse:
        """
        Deletes the browser session associated with the provided browser session ID.
        Requires a valid API key for authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._delete(
            f"/v1/sessions/{session_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponse,
        )

    async def copy(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionCopyResponse:
        """
        Copies the currently selected text to the clipboard

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/copy",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionCopyResponse,
        )

    async def drag_and_drop(
        self,
        session_id: str,
        *,
        end_x: int,
        end_y: int,
        start_x: int,
        start_y: int,
        button: Literal["left", "middle", "right"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionDragAndDropResponse:
        """
        Performs a drag and drop operation from start coordinates to end coordinates

        Args:
          end_x: Ending X coordinate

          end_y: Ending Y coordinate

          start_x: Starting X coordinate

          start_y: Starting Y coordinate

          button: Mouse button to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/drag-and-drop",
            body=await async_maybe_transform(
                {
                    "end_x": end_x,
                    "end_y": end_y,
                    "start_x": start_x,
                    "start_y": start_y,
                    "button": button,
                },
                session_drag_and_drop_params.SessionDragAndDropParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionDragAndDropResponse,
        )

    async def goto(
        self,
        session_id: str,
        *,
        url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionGotoResponse:
        """
        Navigates the browser session to the specified URL

        Args:
          url: The URL to navigate to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/goto",
            body=await async_maybe_transform({"url": url}, session_goto_params.SessionGotoParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionGotoResponse,
        )

    async def list_pages(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionListPagesResponse:
        """
        Retrieves a list of pages associated with a specific browser session.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v1/sessions/{session_id}/pages",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionListPagesResponse,
        )

    async def paste(
        self,
        session_id: str,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionPasteResponse:
        """
        Pastes text at the current cursor position

        Args:
          text: Text to paste

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/paste",
            body=await async_maybe_transform({"text": text}, session_paste_params.SessionPasteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionPasteResponse,
        )

    async def retrieve_downloads(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionRetrieveDownloadsResponse:
        """Retrieves metadata of files downloaded during a browser session.

        Requires a
        valid API key for authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v1/sessions/{session_id}/downloads",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionRetrieveDownloadsResponse,
        )

    async def retrieve_screenshot(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Takes a screenshot of the current browser session and returns it as an image.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        extra_headers = {"Accept": "image/png", **(extra_headers or {})}
        return await self._get(
            f"/v1/sessions/{session_id}/screenshot",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def scroll(
        self,
        session_id: str,
        *,
        delta_y: int,
        x: int,
        y: int,
        delta_x: int | Omit = omit,
        steps: int | Omit = omit,
        use_os: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionScrollResponse:
        """
        Performs a scroll action at the specified coordinates

        Args:
          delta_y: Vertical scroll amount (positive is down, negative is up)

          x: X coordinate

          y: Y coordinate

          delta_x: Horizontal scroll amount (positive is right, negative is left)

          steps: Number of steps to break the scroll into for smoother scrolling

          use_os: Whether to use the OS scroll or the Playwright scroll

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v1/sessions/{session_id}/scroll",
            body=await async_maybe_transform(
                {
                    "delta_y": delta_y,
                    "x": x,
                    "y": y,
                    "delta_x": delta_x,
                    "steps": steps,
                    "use_os": use_os,
                },
                session_scroll_params.SessionScrollParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionScrollResponse,
        )

    async def upload_file(
        self,
        session_id: str,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionUploadFileResponse:
        """
        Upload files directly to a browser session for use with web forms and file
        inputs.

        Files are saved to the session's uploads directory and can be referenced in CDP
        commands.

        Args:
          file: File to upload to the browser session

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/v1/sessions/{session_id}/uploads",
            body=await async_maybe_transform(body, session_upload_file_params.SessionUploadFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionUploadFileResponse,
        )


class SessionsResourceWithRawResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.create = to_raw_response_wrapper(
            sessions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            sessions.retrieve,
        )
        self.delete = to_raw_response_wrapper(
            sessions.delete,
        )
        self.copy = to_raw_response_wrapper(
            sessions.copy,
        )
        self.drag_and_drop = to_raw_response_wrapper(
            sessions.drag_and_drop,
        )
        self.goto = to_raw_response_wrapper(
            sessions.goto,
        )
        self.list_pages = to_raw_response_wrapper(
            sessions.list_pages,
        )
        self.paste = to_raw_response_wrapper(
            sessions.paste,
        )
        self.retrieve_downloads = to_raw_response_wrapper(
            sessions.retrieve_downloads,
        )
        self.retrieve_screenshot = to_custom_raw_response_wrapper(
            sessions.retrieve_screenshot,
            BinaryAPIResponse,
        )
        self.scroll = to_raw_response_wrapper(
            sessions.scroll,
        )
        self.upload_file = to_raw_response_wrapper(
            sessions.upload_file,
        )

    @cached_property
    def all(self) -> AllResourceWithRawResponse:
        return AllResourceWithRawResponse(self._sessions.all)

    @cached_property
    def recordings(self) -> RecordingsResourceWithRawResponse:
        return RecordingsResourceWithRawResponse(self._sessions.recordings)

    @cached_property
    def mouse(self) -> MouseResourceWithRawResponse:
        return MouseResourceWithRawResponse(self._sessions.mouse)

    @cached_property
    def keyboard(self) -> KeyboardResourceWithRawResponse:
        return KeyboardResourceWithRawResponse(self._sessions.keyboard)

    @cached_property
    def clipboard(self) -> ClipboardResourceWithRawResponse:
        return ClipboardResourceWithRawResponse(self._sessions.clipboard)

    @cached_property
    def agent(self) -> AgentResourceWithRawResponse:
        return AgentResourceWithRawResponse(self._sessions.agent)


class AsyncSessionsResourceWithRawResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.create = async_to_raw_response_wrapper(
            sessions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            sessions.retrieve,
        )
        self.delete = async_to_raw_response_wrapper(
            sessions.delete,
        )
        self.copy = async_to_raw_response_wrapper(
            sessions.copy,
        )
        self.drag_and_drop = async_to_raw_response_wrapper(
            sessions.drag_and_drop,
        )
        self.goto = async_to_raw_response_wrapper(
            sessions.goto,
        )
        self.list_pages = async_to_raw_response_wrapper(
            sessions.list_pages,
        )
        self.paste = async_to_raw_response_wrapper(
            sessions.paste,
        )
        self.retrieve_downloads = async_to_raw_response_wrapper(
            sessions.retrieve_downloads,
        )
        self.retrieve_screenshot = async_to_custom_raw_response_wrapper(
            sessions.retrieve_screenshot,
            AsyncBinaryAPIResponse,
        )
        self.scroll = async_to_raw_response_wrapper(
            sessions.scroll,
        )
        self.upload_file = async_to_raw_response_wrapper(
            sessions.upload_file,
        )

    @cached_property
    def all(self) -> AsyncAllResourceWithRawResponse:
        return AsyncAllResourceWithRawResponse(self._sessions.all)

    @cached_property
    def recordings(self) -> AsyncRecordingsResourceWithRawResponse:
        return AsyncRecordingsResourceWithRawResponse(self._sessions.recordings)

    @cached_property
    def mouse(self) -> AsyncMouseResourceWithRawResponse:
        return AsyncMouseResourceWithRawResponse(self._sessions.mouse)

    @cached_property
    def keyboard(self) -> AsyncKeyboardResourceWithRawResponse:
        return AsyncKeyboardResourceWithRawResponse(self._sessions.keyboard)

    @cached_property
    def clipboard(self) -> AsyncClipboardResourceWithRawResponse:
        return AsyncClipboardResourceWithRawResponse(self._sessions.clipboard)

    @cached_property
    def agent(self) -> AsyncAgentResourceWithRawResponse:
        return AsyncAgentResourceWithRawResponse(self._sessions.agent)

class SessionsResourceWithStreamingResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.create = to_streamed_response_wrapper(
            sessions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            sessions.retrieve,
        )
        self.delete = to_streamed_response_wrapper(
            sessions.delete,
        )
        self.copy = to_streamed_response_wrapper(
            sessions.copy,
        )
        self.drag_and_drop = to_streamed_response_wrapper(
            sessions.drag_and_drop,
        )
        self.goto = to_streamed_response_wrapper(
            sessions.goto,
        )
        self.list_pages = to_streamed_response_wrapper(
            sessions.list_pages,
        )
        self.paste = to_streamed_response_wrapper(
            sessions.paste,
        )
        self.retrieve_downloads = to_streamed_response_wrapper(
            sessions.retrieve_downloads,
        )
        self.retrieve_screenshot = to_custom_streamed_response_wrapper(
            sessions.retrieve_screenshot,
            StreamedBinaryAPIResponse,
        )
        self.scroll = to_streamed_response_wrapper(
            sessions.scroll,
        )
        self.upload_file = to_streamed_response_wrapper(
            sessions.upload_file,
        )

    @cached_property
    def all(self) -> AllResourceWithStreamingResponse:
        return AllResourceWithStreamingResponse(self._sessions.all)

    @cached_property
    def recordings(self) -> RecordingsResourceWithStreamingResponse:
        return RecordingsResourceWithStreamingResponse(self._sessions.recordings)

    @cached_property
    def mouse(self) -> MouseResourceWithStreamingResponse:
        return MouseResourceWithStreamingResponse(self._sessions.mouse)

    @cached_property
    def keyboard(self) -> KeyboardResourceWithStreamingResponse:
        return KeyboardResourceWithStreamingResponse(self._sessions.keyboard)

    @cached_property
    def clipboard(self) -> ClipboardResourceWithStreamingResponse:
        return ClipboardResourceWithStreamingResponse(self._sessions.clipboard)

    @cached_property
    def agent(self) -> AgentResourceWithStreamingResponse:
        return AgentResourceWithStreamingResponse(self._sessions.agent)

class AsyncSessionsResourceWithStreamingResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.create = async_to_streamed_response_wrapper(
            sessions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            sessions.retrieve,
        )
        self.delete = async_to_streamed_response_wrapper(
            sessions.delete,
        )
        self.copy = async_to_streamed_response_wrapper(
            sessions.copy,
        )
        self.drag_and_drop = async_to_streamed_response_wrapper(
            sessions.drag_and_drop,
        )
        self.goto = async_to_streamed_response_wrapper(
            sessions.goto,
        )
        self.list_pages = async_to_streamed_response_wrapper(
            sessions.list_pages,
        )
        self.paste = async_to_streamed_response_wrapper(
            sessions.paste,
        )
        self.retrieve_downloads = async_to_streamed_response_wrapper(
            sessions.retrieve_downloads,
        )
        self.retrieve_screenshot = async_to_custom_streamed_response_wrapper(
            sessions.retrieve_screenshot,
            AsyncStreamedBinaryAPIResponse,
        )
        self.scroll = async_to_streamed_response_wrapper(
            sessions.scroll,
        )
        self.upload_file = async_to_streamed_response_wrapper(
            sessions.upload_file,
        )

    @cached_property
    def all(self) -> AsyncAllResourceWithStreamingResponse:
        return AsyncAllResourceWithStreamingResponse(self._sessions.all)

    @cached_property
    def recordings(self) -> AsyncRecordingsResourceWithStreamingResponse:
        return AsyncRecordingsResourceWithStreamingResponse(self._sessions.recordings)

    @cached_property
    def mouse(self) -> AsyncMouseResourceWithStreamingResponse:
        return AsyncMouseResourceWithStreamingResponse(self._sessions.mouse)

    @cached_property
    def keyboard(self) -> AsyncKeyboardResourceWithStreamingResponse:
        return AsyncKeyboardResourceWithStreamingResponse(self._sessions.keyboard)

    @cached_property
    def clipboard(self) -> AsyncClipboardResourceWithStreamingResponse:
        return AsyncClipboardResourceWithStreamingResponse(self._sessions.clipboard)

    @cached_property
    def agent(self) -> AsyncAgentResourceWithStreamingResponse:
        return AsyncAgentResourceWithStreamingResponse(self._sessions.agent)


from typing import TYPE_CHECKING, Any, Dict, Callable, Optional, TypedDict
from contextlib import contextmanager, asynccontextmanager, _GeneratorContextManager, _AsyncGeneratorContextManager

from pydantic import BaseModel
from playwright.sync_api import Page, Worker, Browser, BrowserContext
from playwright.async_api import (
    Page as AsyncPage,
    Worker as AsyncWorker,
    Browser as AsyncBrowser,
    BrowserContext as AsyncBrowserContext,
)

if TYPE_CHECKING:
    from collections import Generator, AsyncGenerator


@contextmanager
def get_playwright_chromium_from_cdp_url(
    api_base_url: str, session_id: str, api_key: str
) -> "Generator[Browser, Any, None]":
    from playwright.sync_api import sync_playwright

    browser = None
    playwright = sync_playwright().start()
    try:
        browser = playwright.chromium.connect_over_cdp(get_cdp_url(api_base_url, session_id, api_key))
        yield browser
    finally:
        if browser:
            browser.close()
        playwright.stop()


@asynccontextmanager
async def get_async_playwright_chromium_from_cdp_url(
    api_base_url: str, session_id: str, api_key: str
) -> "AsyncGenerator[AsyncBrowser, None]":
    from playwright.async_api import async_playwright

    browser = None
    playwright = await async_playwright().start()
    try:
        browser = await playwright.chromium.connect_over_cdp(get_cdp_url(api_base_url, session_id, api_key))
        yield browser
    finally:
        if browser:
            await browser.close()
        await playwright.stop()


def get_cdp_url(api_base_url: str, session_id: str, api_key: str) -> str:
    return f"{api_base_url.replace('https://', 'wss://').replace('api.', 'connect.')}?apiKey={api_key}&sessionId={session_id}"


def get_agent_ws_url(api_base_url: str, session_id: str) -> str:
    return f"{api_base_url.replace('https://', 'wss://')}/ws?sessionId={session_id}"


def get_ai_service_worker(browser_context: "BrowserContext") -> Optional["Worker"]:
    return next(
        (
            sw
            for sw in browser_context.service_workers
            if "chrome-extension://bppehibnhionalpjigdjdilknbljaeai/background.js" in sw.url
        ),
        None,
    )


async def get_ai_service_worker_async(browser_context: "AsyncBrowserContext") -> Optional["AsyncWorker"]:
    return next(
        (
            sw
            for sw in browser_context.service_workers
            if "chrome-extension://bppehibnhionalpjigdjdilknbljaeai/background.js" in sw.url
        ),
        None,
    )


class BrowserSetup(BaseModel):
    session_id: str
    base_url: str
    api_key: str
    _browser: Optional[Browser] = None
    _async_browser: Optional[AsyncBrowser] = None
    _context_manager: Optional[_GeneratorContextManager[Browser]] = None
    _async_context_manager: Optional[_AsyncGeneratorContextManager[AsyncBrowser]] = None

    async def __aenter__(self) -> "BrowserSetup":
        self._async_context_manager = get_async_playwright_chromium_from_cdp_url(
            self.base_url,
            self.session_id,
            self.api_key,
        )
        self._async_browser = await self._async_context_manager.__aenter__()
        return self

    def __enter__(self) -> "BrowserSetup":
        self._context_manager = get_playwright_chromium_from_cdp_url(
            self.base_url,
            self.session_id,
            self.api_key,
        )
        self._browser = self._context_manager.__enter__()
        return self

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> Optional[bool]:
        if self._context_manager:
            return self._context_manager.__exit__(exc_type, exc_val, exc_tb)
        return None

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> Optional[bool]:
        if self._async_context_manager:
            return await self._async_context_manager.__aexit__(exc_type, exc_val, exc_tb)
        return None

    @property
    def browser_generator(self) -> _GeneratorContextManager[Browser]:
        return get_playwright_chromium_from_cdp_url(
            self.base_url,
            self.session_id,
            self.api_key,
        )

    @property
    def async_browser_generator(self) -> _AsyncGeneratorContextManager[AsyncBrowser]:
        return get_async_playwright_chromium_from_cdp_url(
            self.base_url,
            self.session_id,
            self.api_key,
        )

    @property
    def browser(self) -> Browser:
        if self._browser is None:
            raise RuntimeError("BrowserSetup must be used as a context manager")
        return self._browser

    @property
    async def async_browser(self) -> AsyncBrowser:
        if self._async_browser is None:
            raise RuntimeError("BrowserSetup must be used as a context manager")
        return self._async_browser

    @property
    def context(self) -> BrowserContext:
        return self.browser.contexts[0]

    @property
    async def async_context(self) -> AsyncBrowserContext:
        return (await self.async_browser).contexts[0]

    @property
    def page(self) -> Page:
        return self.context.pages[0]

    @property
    async def async_page(self) -> AsyncPage:
        return (await self.async_context).pages[0]

    @property
    def ai(self) -> Worker:
        ai_service_worker = get_ai_service_worker(self.context)
        if not ai_service_worker:
            raise ValueError("AI service worker not found")
        return ai_service_worker

    @property
    async def async_ai(self) -> AsyncWorker:
        ai_service_worker = await get_ai_service_worker_async(await self.async_context)
        if not ai_service_worker:
            raise ValueError("AI service worker not found")
        return ai_service_worker


class AgentTaskParams(TypedDict, total=False):
    url: Optional[str]
    output_schema: Optional[Dict[str, Any]]
    on_agent_step: Optional[Callable[[str], None]]
    agent: Optional[str]
    highlight_elements: Optional[bool]
    model: Optional[str]
    provider: Optional[str]
    detect_elements: Optional[bool]
    extended_system_message: Optional[str]
    human_intervention: Optional[bool]
    max_steps: Optional[int]
    secret_values: Optional[Dict[str, Any]]


class BrowserTaskResponse(TypedDict):
    session_id: str
    task_result_task: Any
    playwright_browser: Any



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from anchorbrowser import Anchorbrowser, AsyncAnchorbrowser
from anchorbrowser.types import BatchSessionCreateResponse, BatchSessionRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBatchSessions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Anchorbrowser) -> None:
        batch_session = client.batch_sessions.create(
            count=10,
        )
        assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Anchorbrowser) -> None:
        batch_session = client.batch_sessions.create(
            count=10,
            configuration={
                "browser": {
                    "adblock": {"active": True},
                    "captcha_solver": {"active": True},
                    "disable_web_security": {"active": True},
                    "extensions": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "extra_stealth": {"active": True},
                    "fullscreen": {"active": True},
                    "headless": {"active": True},
                    "p2p_download": {"active": True},
                    "popup_blocker": {"active": True},
                    "profile": {
                        "name": "name",
                        "persist": True,
                    },
                    "viewport": {
                        "height": 900,
                        "width": 1440,
                    },
                },
                "integrations": [
                    {
                        "id": "550e8400-e29b-41d4-a716-446655440000",
                        "configuration": {"load_mode": "all"},
                        "type": "1PASSWORD",
                    }
                ],
                "session": {
                    "initial_url": "https://example.com",
                    "live_view": {"read_only": True},
                    "proxy": {
                        "active": True,
                        "city": "city",
                        "country_code": "af",
                        "region": "region",
                        "type": "anchor_proxy",
                    },
                    "recording": {"active": True},
                    "timeout": {
                        "idle_timeout": 10,
                        "max_duration": 300,
                    },
                },
            },
            metadata={
                "project": "bar",
                "environment": "bar",
            },
        )
        assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Anchorbrowser) -> None:
        response = client.batch_sessions.with_raw_response.create(
            count=10,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch_session = response.parse()
        assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Anchorbrowser) -> None:
        with client.batch_sessions.with_streaming_response.create(
            count=10,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch_session = response.parse()
            assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Anchorbrowser) -> None:
        batch_session = client.batch_sessions.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(BatchSessionRetrieveResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Anchorbrowser) -> None:
        response = client.batch_sessions.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch_session = response.parse()
        assert_matches_type(BatchSessionRetrieveResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Anchorbrowser) -> None:
        with client.batch_sessions.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch_session = response.parse()
            assert_matches_type(BatchSessionRetrieveResponse, batch_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `batch_id` but received ''"):
            client.batch_sessions.with_raw_response.retrieve(
                "",
            )


class TestAsyncBatchSessions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncAnchorbrowser) -> None:
        batch_session = await async_client.batch_sessions.create(
            count=10,
        )
        assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncAnchorbrowser) -> None:
        batch_session = await async_client.batch_sessions.create(
            count=10,
            configuration={
                "browser": {
                    "adblock": {"active": True},
                    "captcha_solver": {"active": True},
                    "disable_web_security": {"active": True},
                    "extensions": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                    "extra_stealth": {"active": True},
                    "fullscreen": {"active": True},
                    "headless": {"active": True},
                    "p2p_download": {"active": True},
                    "popup_blocker": {"active": True},
                    "profile": {
                        "name": "name",
                        "persist": True,
                    },
                    "viewport": {
                        "height": 900,
                        "width": 1440,
                    },
                },
                "integrations": [
                    {
                        "id": "550e8400-e29b-41d4-a716-446655440000",
                        "configuration": {"load_mode": "all"},
                        "type": "1PASSWORD",
                    }
                ],
                "session": {
                    "initial_url": "https://example.com",
                    "live_view": {"read_only": True},
                    "proxy": {
                        "active": True,
                        "city": "city",
                        "country_code": "af",
                        "region": "region",
                        "type": "anchor_proxy",
                    },
                    "recording": {"active": True},
                    "timeout": {
                        "idle_timeout": 10,
                        "max_duration": 300,
                    },
                },
            },
            metadata={
                "project": "bar",
                "environment": "bar",
            },
        )
        assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.batch_sessions.with_raw_response.create(
            count=10,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch_session = await response.parse()
        assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.batch_sessions.with_streaming_response.create(
            count=10,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch_session = await response.parse()
            assert_matches_type(BatchSessionCreateResponse, batch_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        batch_session = await async_client.batch_sessions.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(BatchSessionRetrieveResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.batch_sessions.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch_session = await response.parse()
        assert_matches_type(BatchSessionRetrieveResponse, batch_session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.batch_sessions.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch_session = await response.parse()
            assert_matches_type(BatchSessionRetrieveResponse, batch_session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `batch_id` but received ''"):
            await async_client.batch_sessions.with_raw_response.retrieve(
                "",
            )



from __future__ import annotations

from typing import Optional

from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.agent import on_agent_step_sync, create_task_payload, on_agent_step_async
from ..lib.browser import (
    BrowserSetup,
    AgentTaskParams,
    BrowserTaskResponse,
)
from ..types.session_create_params import Session

__all__ = ["AgentResource", "AsyncAgentResource"]


class AgentResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AgentResourceWithStreamingResponse(self)

    def task(
        self,
        prompt: str,
        *,
        session_options: Optional[Session] = None,
        task_options: Optional[AgentTaskParams] = None,
    ) -> str:
        """Execute an AI agent task within a browser session.

        Creates a new browser session and executes the given prompt as an AI agent task.
        The agent can optionally navigate to a specific URL and use a structured output schema.

        Args:
            prompt (str): The task prompt/instruction for the AI agent to execute.
            session_options (Optional[Session], optional): Configuration options for the
                browser session. Defaults to None, which creates a session with default settings.
            task_options (Optional[AgentTaskParams], optional): Additional task configuration

        Returns:
            str: The result of the AI agent task execution.
        """
        session = self._client.sessions.create(session=session_options or {})
        if not session.data or not session.data.id:
            raise ValueError("Failed to create session: No session ID returned")

        with BrowserSetup(
            session_id=session.data.id,
            base_url=str(self._client.base_url),
            api_key=self._client.api_key,
        ) as browser_setup:
            output_schema = None
            url = None
            agent = None
            highlight_elements = None
            model = None
            provider = None
            detect_elements = None
            extended_system_message = None
            human_intervention = None
            max_steps = None
            secret_values = None
            if task_options:
                output_schema = task_options.get("output_schema")
                url = task_options.get("url")
                agent = task_options.get("agent")
                highlight_elements = task_options.get("highlight_elements")
                model = task_options.get("model")
                provider = task_options.get("provider")
                detect_elements = task_options.get("detect_elements")
                extended_system_message = task_options.get("extended_system_message")
                human_intervention = task_options.get("human_intervention")
                max_steps = task_options.get("max_steps")
                secret_values = task_options.get("secret_values")
                
                if url:
                    browser_setup.page.goto(url)
                on_agent_step = task_options.get("on_agent_step")
                if on_agent_step:
                    on_agent_step_sync(on_agent_step, browser_setup)
            task_payload = create_task_payload(
                prompt,
                output_schema=output_schema,
                agent=agent,
                highlight_elements=highlight_elements,
                model=model,
                provider=provider,
                detect_elements=detect_elements,
                extended_system_message=extended_system_message,
                human_intervention=human_intervention,
                max_steps=max_steps,
                secret_values=secret_values,
            )
            task_result = str(browser_setup.ai.evaluate(task_payload))
            return task_result

    def browser_task(
        self,
        prompt: str,
        *,
        session_options: Optional[Session] = None,
        task_options: Optional[AgentTaskParams] = None,
    ) -> BrowserTaskResponse:
        """Execute an AI agent task and return a browser task response with session control.

        Creates a new browser session, executes the given prompt as an AI agent task \n
        returns a object that includes the session ID, task result, and browser instance for continued interaction by the caller. \n
        This method differs from `task()` by returning control of the browser session to the caller rather than automatically closing it after task completion. \n
        Args:
            prompt (str): The task prompt/instruction for the AI agent to execute. \n
            session_options (Optional[Session], optional): Configuration options for the browser session. Defaults to None, which creates a session with default settings. \n
            task_options (Optional[AgentTaskParams], optional): Additional task configuration including: \n
        - output_schema: Schema for structured output formatting
        - url: URL to navigate to before executing the task
        - on_agent_step: Callback function for agent step events
                Defaults to None.

        Returns:
            Response object containing:
                - session_id: The ID of the created browser session
                - task_result_task: The result of the AI agent task execution
                - playwright_browser: Browser instance for continued interaction
        """
        session = self._client.sessions.create(session=session_options or {})
        if not session.data or not session.data.id:
            raise ValueError("Failed to create session: No session ID returned")

        with BrowserSetup(
            session_id=session.data.id,
            base_url=str(self._client.base_url),
            api_key=self._client.api_key,
        ) as browser_setup:
            output_schema = None
            url = None
            agent = None
            highlight_elements = None
            model = None
            provider = None
            detect_elements = None
            extended_system_message = None
            human_intervention = None
            max_steps = None
            secret_values = None
            if task_options:
                output_schema = task_options.get("output_schema")
                url = task_options.get("url")
                agent = task_options.get("agent")
                highlight_elements = task_options.get("highlight_elements")
                model = task_options.get("model")
                provider = task_options.get("provider")
                detect_elements = task_options.get("detect_elements")
                extended_system_message = task_options.get("extended_system_message")
                human_intervention = task_options.get("human_intervention")
                max_steps = task_options.get("max_steps")
                secret_values = task_options.get("secret_values")
                
                if url:
                    browser_setup.page.goto(url)
                on_agent_step = task_options.get("on_agent_step")
                if on_agent_step:
                    on_agent_step_sync(on_agent_step, browser_setup)
            task_payload = create_task_payload(
                prompt,
                output_schema=output_schema,
                agent=agent,
                highlight_elements=highlight_elements,
                model=model,
                provider=provider,
                detect_elements=detect_elements,
                extended_system_message=extended_system_message,
                human_intervention=human_intervention,
                max_steps=max_steps,
                secret_values=secret_values,
            )
            task_result = str(browser_setup.ai.evaluate(task_payload))
            return BrowserTaskResponse(
                session_id=session.data.id,
                task_result_task=task_result,
                playwright_browser=browser_setup.browser_generator,
            )


class AsyncAgentResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncAgentResourceWithStreamingResponse(self)

    async def task(
        self,
        prompt: str,
        *,
        session_options: Optional[Session] = None,
        task_options: Optional[AgentTaskParams] = None,
    ) -> str:
        """Execute an AI agent task within a browser session.

        Creates a new browser session and executes the given prompt as an AI agent task.
        The agent can optionally navigate to a specific URL and use a structured output schema.

        Args:
            prompt (str): The task prompt/instruction for the AI agent to execute.
            session_options (Optional[Session], optional): Configuration options for the
                browser session. Defaults to None, which creates a session with default settings.
            task_options (Optional[AgentTaskParams], optional): Additional task configuration

        Returns:
            str: The result of the AI agent task execution.
        """
        session = await self._client.sessions.create(session=session_options or {})
        if not session.data or not session.data.id:
            raise ValueError("Failed to create session: No session ID returned")

        browser_setup = BrowserSetup(
            session_id=session.data.id,
            base_url=str(self._client.base_url),
            api_key=self._client.api_key,
        )

        async with browser_setup:
            output_schema = None
            url = None
            agent = None
            highlight_elements = None
            model = None
            provider = None
            detect_elements = None
            extended_system_message = None
            human_intervention = None
            max_steps = None
            secret_values = None
            if task_options:
                output_schema = task_options.get("output_schema")
                url = task_options.get("url")
                agent = task_options.get("agent")
                highlight_elements = task_options.get("highlight_elements")
                model = task_options.get("model")
                provider = task_options.get("provider")
                detect_elements = task_options.get("detect_elements")
                extended_system_message = task_options.get("extended_system_message")
                human_intervention = task_options.get("human_intervention")
                max_steps = task_options.get("max_steps")
                secret_values = task_options.get("secret_values")
                
                if url:
                    await (await browser_setup.async_page).goto(url)
                on_agent_step = task_options.get("on_agent_step")
                if on_agent_step:
                    on_agent_step_async(on_agent_step, browser_setup)
            task_payload = create_task_payload(
                prompt,
                output_schema=output_schema,
                agent=agent,
                highlight_elements=highlight_elements,
                model=model,
                provider=provider,
                detect_elements=detect_elements,
                extended_system_message=extended_system_message,
                human_intervention=human_intervention,
                max_steps=max_steps,
                secret_values=secret_values,
            )
            task_result = await (await browser_setup.async_ai).evaluate(task_payload)
            return str(task_result)

    async def browser_task(
        self,
        prompt: str,
        *,
        session_options: Optional[Session] = None,
        task_options: Optional[AgentTaskParams] = None,
    ) -> BrowserTaskResponse:
        """Execute an AI agent task and return a browser task response with session control.

        Creates a new browser session, executes the given prompt as an AI agent task \n
        returns a object that includes the session ID, task result, and browser instance for continued interaction by the caller. \n
        This method differs from `task()` by returning control of the browser session to the caller rather than automatically closing it after task completion. \n
        Args:
            prompt (str): The task prompt/instruction for the AI agent to execute. \n
            session_options (Optional[Session], optional): Configuration options for the browser session. Defaults to None, which creates a session with default settings. \n
            task_options (Optional[AgentTaskParams], optional): Additional task configuration including: \n
        - output_schema: Schema for structured output formatting
        - url: URL to navigate to before executing the task
        - on_agent_step: Callback function for agent step events
                Defaults to None.

        Returns:
            Response object containing:
                - session_id: The ID of the created browser session
                - task_result_task: The result of the AI agent task execution
                - playwright_browser: Browser instance for continued interaction
        """
        session = await self._client.sessions.create(session=session_options or {})
        if not session.data or not session.data.id:
            raise ValueError("Failed to create session: No session ID returned")

        async with BrowserSetup(
            session_id=session.data.id,
            base_url=str(self._client.base_url),
            api_key=self._client.api_key,
        ) as browser_setup:
            output_schema = None
            url = None
            agent = None
            highlight_elements = None
            model = None
            provider = None
            detect_elements = None
            extended_system_message = None
            human_intervention = None
            max_steps = None
            secret_values = None
            if task_options:
                output_schema = task_options.get("output_schema")
                url = task_options.get("url")
                agent = task_options.get("agent")
                highlight_elements = task_options.get("highlight_elements")
                model = task_options.get("model")
                provider = task_options.get("provider")
                detect_elements = task_options.get("detect_elements")
                extended_system_message = task_options.get("extended_system_message")
                human_intervention = task_options.get("human_intervention")
                max_steps = task_options.get("max_steps")
                secret_values = task_options.get("secret_values")
                
                if url:
                    await (await browser_setup.async_page).goto(url)
                on_agent_step = task_options.get("on_agent_step")
                if on_agent_step:
                    on_agent_step_async(on_agent_step, browser_setup)
            task_payload = create_task_payload(
                prompt,
                output_schema=output_schema,
                agent=agent,
                highlight_elements=highlight_elements,
                model=model,
                provider=provider,
                detect_elements=detect_elements,
                extended_system_message=extended_system_message,
                human_intervention=human_intervention,
                max_steps=max_steps,
                secret_values=secret_values,
            )
            task_result = await (await browser_setup.async_ai).evaluate(task_payload)
            return BrowserTaskResponse(
                session_id=session.data.id,
                task_result_task=task_result,
                playwright_browser=browser_setup.async_browser_generator,
            )


class AgentResourceWithRawResponse:
    def __init__(self, agent: AgentResource) -> None:
        self._agent = agent

        self.task = to_raw_response_wrapper(
            agent.task,
        )
        self.browser_task = to_raw_response_wrapper(
            agent.browser_task,
        )


class AsyncAgentResourceWithRawResponse:
    def __init__(self, agent: AsyncAgentResource) -> None:
        self._agent = agent

        self.task = async_to_raw_response_wrapper(
            agent.task,
        )
        self.browser_task = async_to_raw_response_wrapper(
            agent.browser_task,
        )


class AgentResourceWithStreamingResponse:
    def __init__(self, agent: AgentResource) -> None:
        self._agent = agent

        self.task = to_streamed_response_wrapper(
            agent.task,
        )
        self.browser_task = to_streamed_response_wrapper(
            agent.browser_task,
        )


class AsyncAgentResourceWithStreamingResponse:
    def __init__(self, agent: AsyncAgentResource) -> None:
        self._agent = agent

        self.task = async_to_streamed_response_wrapper(
            agent.task,
        )
        self.browser_task = async_to_streamed_response_wrapper(
            agent.browser_task,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["MouseMoveParams"]


class MouseMoveParams(TypedDict, total=False):
    x: Required[int]
    """X coordinate"""

    y: Required[int]
    """Y coordinate"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .extension_manifest import ExtensionManifest

__all__ = ["ExtensionRetrieveResponse"]


class ExtensionRetrieveResponse(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the extension"""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)
    """Timestamp when the extension was created"""

    manifest: Optional[ExtensionManifest] = None

    name: Optional[str] = None
    """Extension name"""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)
    """Timestamp when the extension was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "SessionCreateParams",
    "Browser",
    "BrowserAdblock",
    "BrowserCaptchaSolver",
    "BrowserDisableWebSecurity",
    "BrowserExtraStealth",
    "BrowserFullscreen",
    "BrowserHeadless",
    "BrowserP2pDownload",
    "BrowserPopupBlocker",
    "BrowserProfile",
    "BrowserViewport",
    "Integration",
    "IntegrationConfiguration",
    "IntegrationConfigurationOnePasswordAllSecretsConfig",
    "IntegrationConfigurationOnePasswordSpecificSecretsConfig",
    "Session",
    "SessionLiveView",
    "SessionProxy",
    "SessionProxyAnchorProxy",
    "SessionProxyCustomProxy",
    "SessionRecording",
    "SessionTimeout",
]


class SessionCreateParams(TypedDict, total=False):
    browser: Browser
    """Browser-specific configurations."""

    integrations: Iterable[Integration]
    """Array of integrations to load in the browser session.

    Integrations must be previously created using the Integrations API.
    """

    session: Session
    """Session-related configurations."""


class BrowserAdblock(TypedDict, total=False):
    active: bool
    """Enable or disable ad-blocking. Defaults to `true`."""


class BrowserCaptchaSolver(TypedDict, total=False):
    active: bool
    """Enable or disable captcha-solving.

    Requires proxy to be active. Defaults to `false`.
    """


class BrowserDisableWebSecurity(TypedDict, total=False):
    active: bool
    """Whether to disable web security features (CORS, same-origin policy, etc.).

    Allows accessing iframes and resources from different origins. Defaults to
    `false`.
    """


class BrowserExtraStealth(TypedDict, total=False):
    active: bool
    """Enable or disable extra stealth mode."""


class BrowserFullscreen(TypedDict, total=False):
    active: bool
    """Enable or disable fullscreen mode.

    When enabled, the browser will start in fullscreen mode. Defaults to `false`.
    """


class BrowserHeadless(TypedDict, total=False):
    active: bool
    """Whether browser should be headless or headful. Defaults to `false`."""


class BrowserP2pDownload(TypedDict, total=False):
    active: bool
    """Enable or disable P2P downloads.

    When enabled, the browser will capture downloads for direct data extraction,
    instead of uploading them on Anchor's storage. Defaults to `false`.
    """


class BrowserPopupBlocker(TypedDict, total=False):
    active: bool
    """Blocks popups, including ads and CAPTCHA consent banners.

    Requires adblock to be active. Defaults to `true`.
    """


class BrowserProfile(TypedDict, total=False):
    name: str
    """The name of the profile to be used during the browser session."""

    persist: bool
    """
    Indicates whether the browser session profile data should be saved when the
    browser session ends. Defaults to `false`.
    """


class BrowserViewport(TypedDict, total=False):
    height: int
    """Height of the viewport in pixels. Defaults to `900`."""

    width: int
    """Width of the viewport in pixels. Defaults to `1440`."""


class Browser(TypedDict, total=False):
    adblock: BrowserAdblock
    """Configuration for ad-blocking."""

    captcha_solver: BrowserCaptchaSolver
    """Configuration for captcha-solving."""

    disable_web_security: BrowserDisableWebSecurity
    """Configuration for disabling web security features."""

    extensions: SequenceNotStr[str]
    """Array of extension IDs to load in the browser session.

    Extensions must be previously uploaded using the Extensions API.
    """

    extra_stealth: BrowserExtraStealth
    """
    Configuration for extra stealth mode to enhance browser fingerprinting
    protection.
    """

    fullscreen: BrowserFullscreen
    """Configuration for fullscreen mode."""

    headless: BrowserHeadless
    """Configuration for headless mode."""

    p2p_download: BrowserP2pDownload
    """Configuration for peer-to-peer download capture functionality."""

    popup_blocker: BrowserPopupBlocker
    """Configuration for popup blocking."""

    profile: BrowserProfile
    """Options for managing and persisting browser session profiles."""

    viewport: BrowserViewport
    """Configuration for the browser's viewport size."""


class IntegrationConfigurationOnePasswordAllSecretsConfig(TypedDict, total=False):
    load_mode: Required[Literal["all"]]
    """Load all secrets from 1Password"""


class IntegrationConfigurationOnePasswordSpecificSecretsConfig(TypedDict, total=False):
    load_mode: Required[Literal["specific"]]
    """Load specific secrets from 1Password"""

    secrets: Required[SequenceNotStr[str]]
    """Array of secret references to load"""


IntegrationConfiguration: TypeAlias = Union[
    IntegrationConfigurationOnePasswordAllSecretsConfig, IntegrationConfigurationOnePasswordSpecificSecretsConfig
]


class Integration(TypedDict, total=False):
    id: Required[str]
    """Unique integration ID"""

    configuration: Required[IntegrationConfiguration]

    type: Required[Literal["1PASSWORD"]]
    """Integration type"""


class SessionLiveView(TypedDict, total=False):
    read_only: bool
    """Enable or disable read-only mode for live viewing. Defaults to `false`."""


class SessionProxyAnchorProxy(TypedDict, total=False):
    active: Required[bool]

    """Enable or disable proxy usage. Defaults to `false`."""
    city: str
    """City name for precise geographic targeting.

    Supported for anchor_proxy only. Can only be used when region is also provided.
    """

    country_code: Literal[
        "af",
        "al",
        "dz",
        "ad",
        "ao",
        "as",
        "ag",
        "ar",
        "am",
        "aw",
        "au",
        "at",
        "az",
        "bs",
        "bh",
        "bb",
        "by",
        "be",
        "bz",
        "bj",
        "bm",
        "bo",
        "ba",
        "br",
        "bg",
        "bf",
        "cm",
        "ca",
        "cv",
        "td",
        "cl",
        "co",
        "cg",
        "cr",
        "ci",
        "hr",
        "cu",
        "cy",
        "cz",
        "dk",
        "dm",
        "do",
        "ec",
        "eg",
        "sv",
        "ee",
        "et",
        "fo",
        "fi",
        "fr",
        "gf",
        "pf",
        "ga",
        "gm",
        "ge",
        "de",
        "gh",
        "gi",
        "gr",
        "gd",
        "gp",
        "gt",
        "gg",
        "gn",
        "gw",
        "gy",
        "ht",
        "hn",
        "hu",
        "is",
        "in",
        "ir",
        "iq",
        "ie",
        "il",
        "it",
        "jm",
        "jp",
        "jo",
        "kz",
        "kw",
        "kg",
        "lv",
        "lb",
        "ly",
        "li",
        "lt",
        "lu",
        "mk",
        "ml",
        "mt",
        "mq",
        "mr",
        "mx",
        "md",
        "mc",
        "me",
        "ma",
        "nl",
        "nz",
        "ni",
        "ng",
        "no",
        "pk",
        "pa",
        "py",
        "pe",
        "ph",
        "pl",
        "pt",
        "pr",
        "qa",
        "ro",
        "lc",
        "sm",
        "sa",
        "sn",
        "rs",
        "sc",
        "sl",
        "sk",
        "si",
        "so",
        "za",
        "kr",
        "es",
        "sr",
        "se",
        "ch",
        "sy",
        "st",
        "tw",
        "tj",
        "tg",
        "tt",
        "tn",
        "tr",
        "tc",
        "ua",
        "ae",
        "us",
        "uy",
        "uz",
        "ve",
        "ye",
        "bd",
        "bw",
        "bn",
        "bi",
        "kh",
        "cn",
        "dj",
        "gq",
        "sz",
        "fj",
        "hk",
        "id",
        "ke",
        "la",
        "ls",
        "lr",
        "mg",
        "mw",
        "my",
        "mv",
        "mn",
        "mz",
        "mm",
        "na",
        "np",
        "nc",
        "ne",
        "om",
        "pg",
        "ru",
        "rw",
        "ws",
        "sg",
        "ss",
        "lk",
        "sd",
        "tz",
        "th",
        "tl",
        "tm",
        "ug",
        "gb",
        "vu",
        "vn",
        "zm",
        "zw",
        "bt",
        "mu",
    ]
    """Supported country codes ISO 2 lowercase

    **On change make sure to update the Proxy type.**
    """

    region: str
    """
    Region code for more specific geographic targeting. The city parameter can only
    be used when region is also provided.
    """

    type: Literal["anchor_proxy", "anchor_residential", "anchor_mobile", "anchor_gov"]
    """**On change make sure to update the country_code.**"""


class SessionProxyCustomProxy(TypedDict, total=False):
    active: Required[bool]

    password: Required[str]
    """Proxy password"""

    server: Required[str]
    """Proxy server address"""

    type: Required[Literal["custom"]]

    username: Required[str]
    """Proxy username"""


SessionProxy: TypeAlias = Union[SessionProxyAnchorProxy, SessionProxyCustomProxy]


class SessionRecording(TypedDict, total=False):
    active: bool
    """Enable or disable video recording of the browser session. Defaults to `true`."""


class SessionTimeout(TypedDict, total=False):
    idle_timeout: int
    """
    The amount of time (in minutes) the browser session waits for new connections
    after all others are closed before stopping. Defaults to `5`.
    """

    max_duration: int
    """Maximum amount of time (in minutes) for the browser to run before terminating.

    Defaults to `20`.
    """


class Session(TypedDict, total=False):
    initial_url: str
    """The URL to navigate to when the browser session starts.

    If not provided, the browser will load an empty page.
    """

    live_view: SessionLiveView
    """Configuration for live viewing the browser session."""

    proxy: SessionProxy
    """Proxy Documentation available at [Proxy Documentation](/advanced/proxy)"""

    recording: SessionRecording
    """Configuration for session recording."""

    timeout: SessionTimeout
    """Timeout configurations for the browser session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from ..types import task_list_params, task_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.task import run_execute_params
from .._base_client import make_request_options
from ..types.task_list_response import TaskListResponse
from ..types.task_create_response import TaskCreateResponse
from ..types.task.run_execute_response import RunExecuteResponse

__all__ = ["TaskResource", "AsyncTaskResource"]


class TaskResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TaskResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return TaskResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TaskResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return TaskResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        language: Literal["typescript"],
        name: str,
        browser_configuration: task_create_params.BrowserConfiguration | Omit = omit,
        code: str | Omit = omit,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskCreateResponse:
        """Creates a new task or updates an existing task with the same name.

        Tasks are
        reusable code snippets that can be executed in browser sessions. Tasks support
        versioning with draft and published versions.

        Args:
          language: Programming language for the task

          name: Task name (letters, numbers, hyphens, and underscores only)

          browser_configuration: Browser configuration for task execution

          code: Base64 encoded task code (optional)

          description: Optional description of the task

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/task",
            body=maybe_transform(
                {
                    "language": language,
                    "name": name,
                    "browser_configuration": browser_configuration,
                    "code": code,
                    "description": description,
                },
                task_create_params.TaskCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskCreateResponse,
        )

    def list(
        self,
        *,
        limit: str | Omit = omit,
        page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskListResponse:
        """Retrieves a paginated list of all tasks for the authenticated team.

        Tasks are
        returned with their latest version information and metadata.

        Args:
          limit: Number of tasks per page

          page: Page number

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/task",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                    },
                    task_list_params.TaskListParams,
                ),
            ),
            cast_to=TaskListResponse,
        )

    def run(
        self,
        *,
        task_id: str,
        inputs: Dict[str, str] | Omit = omit,
        override_browser_configuration: run_execute_params.OverrideBrowserConfiguration | Omit = omit,
        session_id: str | Omit = omit,
        task_session_id: str | Omit = omit,
        version: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunExecuteResponse:
        """Executes a task in a browser session.

        The task can be run with a specific
        version or the latest version. Optionally, you can provide an existing session
        ID or let the system create a new one.

        Args:
          task_id: Task identifier

          inputs: Environment variables for task execution (keys must start with ANCHOR\\__)

          override_browser_configuration: Override browser configuration for this execution

          session_id: Optional existing session ID to use

          task_session_id: Optional task-specific session ID

          version: Version to run (draft, latest, or version number)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/task/run",
            body=maybe_transform(
                {
                    "task_id": task_id,
                    "inputs": inputs,
                    "override_browser_configuration": override_browser_configuration,
                    "session_id": session_id,
                    "task_session_id": task_session_id,
                    "version": version,
                },
                run_execute_params.RunExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunExecuteResponse,
        )


class AsyncTaskResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTaskResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncTaskResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTaskResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncTaskResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        language: Literal["typescript"],
        name: str,
        browser_configuration: task_create_params.BrowserConfiguration | Omit = omit,
        code: str | Omit = omit,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskCreateResponse:
        """Creates a new task or updates an existing task with the same name.

        Tasks are
        reusable code snippets that can be executed in browser sessions. Tasks support
        versioning with draft and published versions.

        Args:
          language: Programming language for the task

          name: Task name (letters, numbers, hyphens, and underscores only)

          browser_configuration: Browser configuration for task execution

          code: Base64 encoded task code (optional)

          description: Optional description of the task

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/task",
            body=await async_maybe_transform(
                {
                    "language": language,
                    "name": name,
                    "browser_configuration": browser_configuration,
                    "code": code,
                    "description": description,
                },
                task_create_params.TaskCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskCreateResponse,
        )

    async def list(
        self,
        *,
        limit: str | Omit = omit,
        page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskListResponse:
        """Retrieves a paginated list of all tasks for the authenticated team.

        Tasks are
        returned with their latest version information and metadata.

        Args:
          limit: Number of tasks per page

          page: Page number

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/task",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                    },
                    task_list_params.TaskListParams,
                ),
            ),
            cast_to=TaskListResponse,
        )

    async def run(
        self,
        *,
        task_id: str,
        inputs: Dict[str, str] | Omit = omit,
        override_browser_configuration: run_execute_params.OverrideBrowserConfiguration | Omit = omit,
        session_id: str | Omit = omit,
        task_session_id: str | Omit = omit,
        version: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RunExecuteResponse:
        """Executes a task in a browser session.

        The task can be run with a specific
        version or the latest version. Optionally, you can provide an existing session
        ID or let the system create a new one.

        Args:
          task_id: Task identifier

          inputs: Environment variables for task execution (keys must start with ANCHOR\\__)

          override_browser_configuration: Override browser configuration for this execution

          session_id: Optional existing session ID to use

          task_session_id: Optional task-specific session ID

          version: Version to run (draft, latest, or version number)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/task/run",
            body=await async_maybe_transform(
                {
                    "task_id": task_id,
                    "inputs": inputs,
                    "override_browser_configuration": override_browser_configuration,
                    "session_id": session_id,
                    "task_session_id": task_session_id,
                    "version": version,
                },
                run_execute_params.RunExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RunExecuteResponse,
        )


class TaskResourceWithRawResponse:
    def __init__(self, task: TaskResource) -> None:
        self._task = task

        self.create = to_raw_response_wrapper(
            task.create,
        )
        self.list = to_raw_response_wrapper(
            task.list,
        )
        self.run = to_raw_response_wrapper(
            task.run,
        )


class AsyncTaskResourceWithRawResponse:
    def __init__(self, task: AsyncTaskResource) -> None:
        self._task = task

        self.create = async_to_raw_response_wrapper(
            task.create,
        )
        self.list = async_to_raw_response_wrapper(
            task.list,
        )
        self.run = async_to_raw_response_wrapper(
            task.run,
        )


class TaskResourceWithStreamingResponse:
    def __init__(self, task: TaskResource) -> None:
        self._task = task

        self.create = to_streamed_response_wrapper(
            task.create,
        )
        self.list = to_streamed_response_wrapper(
            task.list,
        )
        self.run = to_streamed_response_wrapper(
            task.run,
        )


class AsyncTaskResourceWithStreamingResponse:
    def __init__(self, task: AsyncTaskResource) -> None:
        self._task = task

        self.create = async_to_streamed_response_wrapper(
            task.create,
        )
        self.list = async_to_streamed_response_wrapper(
            task.list,
        )
        self.run = async_to_streamed_response_wrapper(
            task.run,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from ...._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.sessions.agent import file_upload_params
from ....types.sessions.agent.file_list_response import FileListResponse
from ....types.sessions.agent.file_upload_response import FileUploadResponse

__all__ = ["FilesResource", "AsyncFilesResource"]


class FilesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return FilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return FilesResourceWithStreamingResponse(self)

    def list(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FileListResponse:
        """
        List all resources that have been uploaded to the browser session for agent use.
        Returns resource metadata including name, size, type, and last modified
        timestamp.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v1/sessions/{session_id}/agent/files",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileListResponse,
        )

    def upload(
        self,
        session_id: str,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FileUploadResponse:
        """
        Upload files as agent resources to a browser session using multipart/form-data.
        If you upload a ZIP file, it will be automatically extracted and the files will
        be made available as agent resources. If you upload a single file, it will be
        saved directly as an agent resource. Resources are then accessible to AI agents
        for task completion and automation.

        Args:
          file: File to upload as agent resource (ZIP files will be extracted automatically)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/v1/sessions/{session_id}/agent/files",
            body=maybe_transform(body, file_upload_params.FileUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileUploadResponse,
        )


class AsyncFilesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncFilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncFilesResourceWithStreamingResponse(self)

    async def list(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FileListResponse:
        """
        List all resources that have been uploaded to the browser session for agent use.
        Returns resource metadata including name, size, type, and last modified
        timestamp.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v1/sessions/{session_id}/agent/files",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileListResponse,
        )

    async def upload(
        self,
        session_id: str,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FileUploadResponse:
        """
        Upload files as agent resources to a browser session using multipart/form-data.
        If you upload a ZIP file, it will be automatically extracted and the files will
        be made available as agent resources. If you upload a single file, it will be
        saved directly as an agent resource. Resources are then accessible to AI agents
        for task completion and automation.

        Args:
          file: File to upload as agent resource (ZIP files will be extracted automatically)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/v1/sessions/{session_id}/agent/files",
            body=await async_maybe_transform(body, file_upload_params.FileUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileUploadResponse,
        )


class FilesResourceWithRawResponse:
    def __init__(self, files: FilesResource) -> None:
        self._files = files

        self.list = to_raw_response_wrapper(
            files.list,
        )
        self.upload = to_raw_response_wrapper(
            files.upload,
        )


class AsyncFilesResourceWithRawResponse:
    def __init__(self, files: AsyncFilesResource) -> None:
        self._files = files

        self.list = async_to_raw_response_wrapper(
            files.list,
        )
        self.upload = async_to_raw_response_wrapper(
            files.upload,
        )


class FilesResourceWithStreamingResponse:
    def __init__(self, files: FilesResource) -> None:
        self._files = files

        self.list = to_streamed_response_wrapper(
            files.list,
        )
        self.upload = to_streamed_response_wrapper(
            files.upload,
        )


class AsyncFilesResourceWithStreamingResponse:
    def __init__(self, files: AsyncFilesResource) -> None:
        self._files = files

        self.list = async_to_streamed_response_wrapper(
            files.list,
        )
        self.upload = async_to_streamed_response_wrapper(
            files.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["BatchSessionCreateResponse", "Data"]


class Data(BaseModel):
    batch_id: Optional[str] = None
    """Unique identifier for the batch"""

    created_at: Optional[datetime] = None
    """Timestamp when the batch was created"""

    status: Optional[Literal["pending", "processing", "completed", "failed"]] = None
    """Current status of the batch"""

    total_requests: Optional[int] = None
    """Total number of sessions requested in the batch"""


class BatchSessionCreateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .primary import (
    PrimaryResource,
    AsyncPrimaryResource,
    PrimaryResourceWithRawResponse,
    AsyncPrimaryResourceWithRawResponse,
    PrimaryResourceWithStreamingResponse,
    AsyncPrimaryResourceWithStreamingResponse,
)
from .recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)

__all__ = [
    "PrimaryResource",
    "AsyncPrimaryResource",
    "PrimaryResourceWithRawResponse",
    "AsyncPrimaryResourceWithRawResponse",
    "PrimaryResourceWithStreamingResponse",
    "AsyncPrimaryResourceWithStreamingResponse",
    "RecordingsResource",
    "AsyncRecordingsResource",
    "RecordingsResourceWithRawResponse",
    "AsyncRecordingsResourceWithRawResponse",
    "RecordingsResourceWithStreamingResponse",
    "AsyncRecordingsResourceWithStreamingResponse",
]
